#install required packages
#install.packages("dfcrm")
#install.packages("nnet")
library(dfcrm)
library(nnet)

#load function 'impbcrm'
impbcrm<-function(y,n,theta,begin,dose.curr){

    d=ndose=length(y)
    ptox.hat = numeric(ndose);
    #skeleton=getprior(halfwidth=0.06, target=theta, nu=ifelse(d %% 2 == 0,d/2,(d+1)/2), nlevel=d, model="empiric")
    skeleton=getprior(halfwidth=0.06, target=theta, nu=begin, nlevel=d, model="empiric")
    
    cl=0.9
    
    avar=2
    bcrmh<-function(a,p,y,n){
      s2=avar
      lik=exp(-0.5*a*a/s2)
      for(j in 1:length(p)){
        pj=p[j]**exp(a)
        lik=lik*pj^y[j]*(1-pj)^(n[j]-y[j]);
      }
      return(lik);
    }
    
    bcrmht<-function(a,p,y,n){
      s2=avar
      lik=a*exp(-0.5*a*a/s2)
      for(j in 1:length(p)){
        pj=p[j]**exp(a)
        lik=lik*pj^y[j]*(1-pj)^(n[j]-y[j]);
      }
      return(lik);
    }
    
    bcrmht2<-function(a,p,y,n){
      s2=avar
      lik=a^2*exp(-0.5*a*a/s2)
      for(j in 1:length(p)){
        pj=p[j]**exp(a)
        lik=lik*pj^y[j]*(1-pj)^(n[j]-y[j]);
      }
      return(lik);
    }
    
    marginal= integrate(bcrmh,lower=-Inf,upper=Inf, p=skeleton, y=y,n=n,abs.tol = 0)$value;
    est=integrate(bcrmht,lower=-10,upper=10, skeleton, y, n,abs.tol = 0)$value/marginal
    e2=integrate(bcrmht2,lower=-10,upper=10, skeleton, y, n,abs.tol = 0)$value/marginal
    
    ptox.hat=skeleton**exp(est)
    post.var=e2-(est)^2
    crit=qnorm(0.5+cl/2)
    lb=est-crit*sqrt(post.var)
    ub=est+crit*sqrt(post.var)
    ptoxL=skeleton**exp(ub)
    ptoxU=skeleton**exp(lb)
    
    
    distance=abs(ptox.hat-theta)
    best=which.is.max(-distance)
    
    
    ifelse(ptoxL[1]>theta,dose.curr<-"STOP STUDY",dose.curr<-min(best,dose.curr+1))
    
    cat("Date and time:                           ", format(Sys.time()), sep="\t",  "\n");
    cat("Skeleton of working model:               ", round(skeleton,2), sep="\t",  "\n");
    cat("Number of DLTs:        	              ", y, sep="\t",   "\n");
    cat("Number of patients evaluated for DLT:    ", n, sep="\t",   "\n");
    cat("Estimated DLT probabilities:             ", round(ptox.hat,2), sep="\t",  "\n");
    cat("Target DLT rate:                         ", theta, sep="\t",  "\n");
    cat("Recommended dose level:                  ", dose.curr, sep="\t",  "\n");
  }
#end function 'impbcrm

#user input
y=c(0,0,1,0,0)
n=c(2,2,4,0,0)
theta=0.25
begin=1
dose.curr=2

impbcrm(y,n,theta,begin,dose.curr)



