#install required packages
#install.packages("dfcrm")
#install.packages("nnet")
library(dfcrm)
library(nnet)

#load function 'simbcrm'
simbcrm<-function(truth,target,cohortsize,ssize,n.stop,ntrial,x1,rseed){
    set.seed(rseed)
    
    d=ndose=length(truth)
   skeleton=getprior(halfwidth=0.06, target=target, nu=x1, nlevel=d, model="empiric")
  
        
    cl=0.9
    ncohort=ssize/cohortsize
    
    
    
    ###Load the function 'bayescrm' 
    bayescrm<-function(truth,skeleton,target,cohortsize,ncohort,x1,n.stop){
      
      
      avar=2
      bcrmh<-function(a,p,y,n){
        s2=avar
        lik=exp(-0.5*a*a/s2)
        for(j in 1:length(p)){
          pj=p[j]**exp(a)
          lik=lik*pj^y[j]*(1-pj)^(n[j]-y[j]);
        }
        return(lik);
      }
      
      bcrmht<-function(a,p,y,n){
        s2=avar
        lik=a*exp(-0.5*a*a/s2)
        for(j in 1:length(p)){
          pj=p[j]**exp(a)
          lik=lik*pj^y[j]*(1-pj)^(n[j]-y[j]);
        }
        return(lik);
      }
      
      bcrmht2<-function(a,p,y,n){
        s2=avar
        lik=a^2*exp(-0.5*a*a/s2)
        for(j in 1:length(p)){
          pj=p[j]**exp(a)
          lik=lik*pj^y[j]*(1-pj)^(n[j]-y[j]);
        }
        return(lik);
      }
      
      
      
      
      ### run a trial 	
      ndose = length(skeleton);   #number of combos
      y=n=rep(0,ndose);  #number of toxicities and number of treated patients at each dose level
      curr = x1;  # current dose level	 
      ptox.hat = numeric(ndose); # estimate of toxicity prob
      dose.select=rep(0,ndose); # a vector of indicators for dose selection
      stop=0; #indicate if trial stops early
      i=1	
      while(i <= ncohort)
      {
       
       
        
         # generate data for a new cohort of patients
        lasttox=rbinom(1,cohortsize,truth[curr])
        y[curr] = y[curr] + lasttox;
        n[curr] = n[curr] + cohortsize;
        
        
      
        
        marginal= integrate(bcrmh,lower=-Inf,upper=Inf, p=skeleton, y=y,n=n,abs.tol = 0)$value;
        est=integrate(bcrmht,lower=-10,upper=10, skeleton, y, n,abs.tol = 0)$value/marginal
        e2=integrate(bcrmht2,lower=-10,upper=10, skeleton, y, n,abs.tol = 0)$value/marginal
        
        ptox.hat=skeleton**exp(est)
        post.var=e2-(est)^2
        crit=qnorm(0.5+cl/2)
        lb=est-crit*sqrt(post.var)
        ub=est+crit*sqrt(post.var)
        ptoxL=skeleton**exp(ub)
        ptoxU=skeleton**exp(lb)
        
        if(ptoxL[1]>target){
          stop=1
          break
        }
        
        #distance=abs(ptox.hat-target)
        if (all(ptox.hat <= target)) {
          best = length(skeleton)
        }
        else if (all(ptox.hat >= target)) {
          best = 1
        }
        else {
          best <- order(abs(ptox.hat - target))[1]
        }
        
       # best=order(abs(ptox.hat - target))[1] #which.is.max(-distance)
        if(sum(lasttox)/cohortsize>=target){
          curr<-min(best, curr)
        }
        else {
          curr<-min(best,curr+1)
        }
        if(any(n>=n.stop)){
          stop<-0
          break
        }
        i=i+1
      }
      if(stop==0){
        dose.select[curr]=dose.select[curr]+1;
      }
      return(list(dose.select=dose.select,tox.data=y,pt.allocation=n,stop=stop))
    }
    ##########'bpocrm' end here
    
    d.select<-tox<-pts<-matrix(nrow=ntrial,ncol=ndose)
    nstop=0
    
    for(i in 1:ntrial){
      result<-bayescrm(truth,skeleton,target,cohortsize,ncohort,x1,n.stop)
      d.select[i,]=result$dose.select
      tox[i,]=result$tox.data
      pts[i,]=result$pt.allocation
      nstop=nstop+result$stop
    }
    cat("Skeleton of working model:        ", round(skeleton,2), sep="\t",  "\n");
    cat("True DLT probability:             ", round(truth,3), sep="\t",  "\n");
    cat("MTD selection percentage:         ", formatC(colMeans(d.select)*100, digits=1, format="f"), sep="\t",  "\n");
    cat("Average number of DLTs:           ", formatC(colMeans(tox), digits=1, format="f"), sep="\t",   "\n");
    cat("Average number of patients:       ", formatC(colMeans(pts), digits=1, format="f"), sep="\t",   "\n");
    cat("Percentage stopped for safety:    ", round(nstop/ntrial*100,3),sep="\t", "\n");
  }
#end function 'simbcrm'

#User input
truth=c(0.01,0.05,0.12,0.25,0.40)
target=0.25
cohortsize=2
ssize=24
n.stop=25
ntrial=1000
x1=1
rseed=580

simbcrm(truth,target,cohortsize,ssize,n.stop,ntrial,x1,rseed)

